//////////////////////////////////////////////////////////////////////////////
//
// PocketHAL
// Copyright 2004 by Thierry Tremblay
//
//////////////////////////////////////////////////////////////////////////////

#ifndef POCKETHAL_DISPLAY_H
#define POCKETHAL_DISPLAY_H

#include <PocketHAL/defs.h>
#include <PocketHAL/rect.h>
#include <PocketHAL/utility.h>



namespace PHAL
{

//////////////////////////////////////////////////////////////////////////////
//
// SwapEffect
//
//////////////////////////////////////////////////////////////////////////////

enum SwapEffect
{
    SWAP_NONE,      // Direct access (no buffering)
    SWAP_DISCARD,   // We don't care about the content of the backbuffer
    SWAP_COPY       // We want to keep the backbuffer content
};



//////////////////////////////////////////////////////////////////////////////
//
// DisplayMode
//
//////////////////////////////////////////////////////////////////////////////

struct DisplayMode
{
    unsigned    m_width;        // Width of display
    unsigned    m_height;       // Height of display
    Orientation m_orientation;  // Orientation
};



//////////////////////////////////////////////////////////////////////////////
//
// DisplayParameters
//
//////////////////////////////////////////////////////////////////////////////

struct DisplayParameters
{
    unsigned    m_width;        // Width of backbuffer
    unsigned    m_height;       // Height of backbuffer
    Orientation m_orientation;  // Orientation
    SwapEffect  m_swapEffect;   // Swap effect

    // Construction
    DisplayParameters( Orientation orientation = ORIENTATION_NORMAL,
                       SwapEffect  swapEffect  = SWAP_DISCARD )
        :   m_width(0),
            m_height(0),
            m_orientation( orientation ),
            m_swapEffect( swapEffect )
        {}
};



//////////////////////////////////////////////////////////////////////////////
//
// Flags for Display::Swap()
//
//////////////////////////////////////////////////////////////////////////////

enum
{
    SWAP_VSYNC       = 1,   // Swap on VSYNC if possible, otherwise minimize tearing
    SWAP_FULL_UPDATE = 2,   // Hint that all pixels need to be updated
};



//////////////////////////////////////////////////////////////////////////////
//
// Display
//
//////////////////////////////////////////////////////////////////////////////

class Surface;

class Display : NonCopyable
{
public:

    //////////////////////////////////////////////////////////////////////////
    // Static interface 
    //////////////////////////////////////////////////////////////////////////

    // Create a display object
    static Display* Create( WindowHandle hWindow );
    static Display* Create( WindowHandle hWindow, DisplayParameters& params );

    // Return the number of display modes available
    static unsigned GetModeCount();

    // Return a description of display mode 'index'
    static bool EnumMode( unsigned index, DisplayMode* mode );



    //////////////////////////////////////////////////////////////////////////
    // Class interface
    //////////////////////////////////////////////////////////////////////////

    // Retrieve the display parameters in use
    const DisplayParameters& GetParameters() const;

    // Destructor
    ~Display();

    // Begin a scene. Must be called before rendering. This function will fail if your
    // window doesn't have focus. Note that GetBackBuffer() will return 0 if BeginScene()
    // fails or is not called first.
    bool BeginScene();

    // Return the backbuffer (will return 0 if BeginScene() hasn't been called).
    Surface* GetBackBuffer();

    // Set the update viewport
    void SetViewport( const Rect* viewport );

    // Update the display with the content of the backbuffer (i.e. flip).
    // Will fail if BeginScene() or GetBackBuffer() wasn't called.
    bool Swap( unsigned flags = 0 );



#ifdef PHAL_PLATFORM_SYMBIAN
    // Your application must call this when it's foreground status changes
    void HandleForegroundEvent( TBool aForeground );
#endif


    // Implementation details
public:
    class Impl;

private:
    Display( Impl* );
    Impl* m_impl;
};



//////////////////////////////////////////////////////////////////////////
// Implementation
//////////////////////////////////////////////////////////////////////////

inline Display* Display::Create( WindowHandle hWindow )
{
    DisplayParameters params;
    return Create( hWindow, params );
}



} // end of namespace PHAL



#endif
